import torch
from audiomentations import AddShortNoises, PolarityInversion, AddBackgroundNoise, TanhDistortion, AddGaussianNoise, AddGaussianSNR
from random import shuffle, random

from shttst.audio.utils import load_audio, save_audio

def get_noise_transform(noise_dir="/content/MUSAN"):
    transforms = [AddShortNoises(
        sounds_path=noise_dir,
        min_snr_in_db=15.0,
        max_snr_in_db=25.0,
        noise_rms="relative_to_whole_input",
        min_time_between_sounds=2.0,
        max_time_between_sounds=8.0,
        noise_transform=PolarityInversion(),
        p=1.0
    ), AddBackgroundNoise(
        sounds_path=noise_dir,
        min_snr_in_db=15.0,
        max_snr_in_db=25.0,
        noise_transform=PolarityInversion(),
        p=1.0
    ), TanhDistortion(
        min_distortion=0.45,
        max_distortion=0.85,
        p=1.0
    ), AddGaussianNoise(
        min_amplitude=0.001,
        max_amplitude=0.015,
        p=1.0
    ), AddGaussianSNR(
        min_snr_in_db=15.0,
        max_snr_in_db=35.0,
        p=1.0
    )]
    shuffle(transforms)
    return transforms[0]

def apply_random_noise(audio_tensor, sr=22050, noise_dir="/content/MUSAN"):
    return torch.FloatTensor(get_noise_transform(noise_dir)(audio_tensor[0].numpy(), sr)).unsqueeze(0)

if __name__ == '__main__':
    wave, sr = load_audio('/content/manual/Bezi/wavs/DIA_BaalIsidro_GimmeKraut_15_00.wav')
    save_audio('/content/noise.wav', apply_random_noise(wave, sr))


import os
import shutil
from typing import List
from tqdm import tqdm

from shttst.audio import classify_audio_file, get_transcription
from shttst.dataset import ShmartTranscriptionLine
from shttst.files import append_file, destroy_file, get_audio_files

def classify_step(dir_path, audio_list, remove_bad=False) -> List[str]:
    ok=[]
    for path in tqdm(audio_list):
        audio_class = classify_audio_file(path)
        if  audio_class != 'fine':
            if remove_bad:
                destroy_file(path)
            else:
                append_file(os.path.join(dir_path, '1_bad.txt'), f'{path}|{audio_class}\n')
        else:
            speaker_path = os.path.dirname(path)
            if not speaker_path.endswith('\\wavs'):
                wavs_path = os.path.join(speaker_path, 'wavs')
                os.makedirs(wavs_path)
                new_path = os.path.join(wavs_path, os.path.basename(path))
                shutil.move(path, new_path)
                path = new_path
            
            append_file(os.path.join(dir_path, '1_good.txt'), f'{path}\n')
            ok.append(path)
    return ok
        
def transcribe_directory(dir_path: str, min_chars_per_sec=9, max_chars_per_sec=24, classify=True, remove_bad=False) -> None:
    audio_list = get_audio_files(dir_path)
    if classify:
        audio_list = classify_step(dir_path, audio_list, remove_bad)
    
    for path in tqdm(audio_list):
        wavs_path = os.path.dirname(path)
        text, duration = get_transcription(path)
        chars_per_sec = len(text) / duration
        if not text or chars_per_sec < min_chars_per_sec or chars_per_sec > max_chars_per_sec:
            append_file(os.path.join(dir_path, '2_bad.txt'), f'{path}|{text}|{duration}\n')
            continue

        line = ShmartTranscriptionLine(
            os.path.basename(path).replace('.wav', ''),
            text,
            'unknown',
            'normal',
            duration
        )
        append_file(os.path.join(wavs_path, os.path.pardir, 'metadata.csv'), f'{line.serialize()}')
        

if __name__ == '__main__':
    transcribe_directory('/content/anon', remove_bad=True)


from typing import List
from shttst.text import shmart_cleaner, symbols

class ShmartTextProcessor():
    def __init__(self, cleaner_fn=shmart_cleaner, symbols: List[str] = symbols) -> None:
        self.cleaner = cleaner_fn

        self.symbol_to_id_map = {s: i for i, s in enumerate(symbols)}
        self.id_to_symbol_map = {i: s for i, s in enumerate(symbols)}

    def encode_text(self, text: str) -> List[int]:
        space_id = self.symbol_to_id_map[' ']
        return [space_id, *self._symbols_to_sequence(self._clean_text(text)), space_id]

    def _clean_text(self, text: str):
        return self.cleaner(text)

    def _symbols_to_sequence(self, symbols: str):
        return [self.symbol_to_id_map[s] for s in symbols if s in self.symbol_to_id_map]


from typing import List, Tuple
from shttst.text.shmart_nums import normalize_numbers
import re

rules_def: List[Tuple[str, str]] = [
    ('\'', ''),
    ('”', '"'),
    ('„', '"'),
    ('–', '-'),
    ('—', '-'),
    ('...', '…'),
    ('»', '"'),
    ('’', "'"),
    ('«', '"'),
    ('\(', '"'),
    ('\)', '"'),
    ('odziem', 'od_ziem'),
    ('ęzis', 'ęz_is'),
    ('arzn', 'ar_zn'),
    ('arzl', 'ar_zl'),
    ('arzł', 'ar_zł'),
    ('asilo', 'as_ilo'),
    (' tarzan', ' tar_zan'),
    ('kontratak', 'kontr_atak'),
    ('mierzi', 'mier_zi'),
    ('mierzł', 'mier_zł'),
    ('odzown', 'od_zown'),
    ('zinte', 'z_inte'),
    (' zign', ' z_ign'),
    (' bezi', ' be_zi'),
    (' ziry', ' z_iry'),
    ('zide', 'z_ide'),
    ('ziszcz', 'z_iszcz'),
    (' nadzor', ' nad_zor'),
    ('hunter', 'hanter'),
    (' slums', ' slams'),
    (' bruce', ' brus'),
    (' hulk', ' halk'),
    ('she-hulk', 'szi-halk'),
    ('lady-hulk', 'lejdi-halk'),
    ('girl-hulk', 'gerl-halk'),
    (' walter', ' łolter'),
    (' steve', ' stiw'),
    (' roger', ' rodżer'),
    ('jennifer', 'dżenifer'),
    (' jen ', ' dżen '),
    (' yen ', ' jen '),
    ('cheetos', 'czitos'),
    (' holiway', 'holiłej'),
    (' blonsky', 'blonski'),
    ('carring', 'karing'),
    ('izaac', 'izaak'),
    ('sprite', 'sprajt'),
    ('avonlea', 'avonli'),
    (' chubb', 'czab'),
    (' nigel', ' najdżel'),
    ('lady ', 'lejdi '), #unsafe
    ('pauline', 'polin'),
    (' jerome', ' dżerom'),
    ('rodney', 'rodnyj'),
    ('morgarath', 'morgaraf'),
    (' horace', ' horys'),
    (' mount ', ' maunt '),
    ('beethoven', 'betoven'),
    ('mozart', 'mocart'),
    (' howard', ' hałard'),
    (' clara', ' klara'),
    (' duncan', ' dankan'),
    (' hackham', ' hakam'),
    ('slipsunder', 'slipsander'),
    (' george', 'dżordż'),
    (' sir ', ' ser '),
    (' jenny', ' dżeny'),
    (' rosie ', ' roz_i '),
    (' melanie ', ' melani '),
    (' caldwell ', ' kaldłel '),
    (' kitty ', ' ki_ti '),
    (' sarah ', ' sara '),
    ('cruiser', 'kruzer'),
    (' cushing ', ' kaszing '),
    (' chandler ', ' czendler '),
    (' newport ', ' niuport '),
    (' david ', ' dejwid '),
    (' fielding ', ' filding '),
    ('antoinette', 'antłenet'),
    ('wallace', 'łoles'),
    ('permettez-moi', 'per-me-te-mła'),
    ('carter', 'karter'),
    ('cordom', 'kordom'),
    ('wensley', 'łenzli'),
    ('casting', 'kasting'),
    ('klermont', 'klermąn'),
    ('melbourne', 'melbern'),
    ('shirley', 'szirlij'),
    (' alice', ' alis'),
    ('auschwitz', 'ałszwic'),
    ('beethoven', 'betowen'),
    ('benjamin', 'bendżamin'),
    ('birmingham', 'birmingam'),
    ('bismarck', 'bismark'),
    (' blake', ' blejk'),
    ('brewster', 'bruster'),
    ('brooklyn', 'bruklin'),
    (' brown', ' brałn'),
    (' bush', ' busz'),
    (' carlos', ' karlos'),
    ('castaneda', 'kastaneda'),
    (' carson', ' karson'),
    (' carter', ' karter'),
    ('catalina', ' katalina'),
    ('charles', 'czarls'),
    ('charlie', 'czarli'),
    ('charlotte', 'szarlot'),
    (' chase', ' czejs'),
    ('chelsea', 'czels_i'),
    (' chopin', ' szopen'),
    (' colorado', ' kolorado'),
    (' columb', ' kolumb'),
    ('connor', 'konor'),
    ('conrad', 'konrad'),
    ('crusoe', 'kruzo'),
    (' duncan', ' dankan'),
    ('eiffla', 'ajfla'),
    ('everest', 'ewrest'),
    ('facebook', 'fejsbuk'),
    (' federic', ' federik'),
    (' freya', ' freja'),
    (' gray', ' grej'),
    ('greenwich', 'grinłicz'),
    (' grey', ' grej'),
    ('isabel', 'izabel'),
    ('jacob', ' dżejkob'),
    (' james', ' dżejms'),
    (' jeremy', ' dżeremi'),
    (' jenkins', ' dżenkins'),
    (' jim', ' dżim'),
    (' joe ', ' dżo '),
    (' john', ' dżon'),
    (' justin', ' dżastin'),
    ('liverpool', 'liwerpul'),
    ('lorraine', 'loren'),
    (' louis', ' lu_łiz'),
    (' lucy ', ' lus_i '),
    ('malcolm', 'malkolm'),
    ('massachusetts', 'masaczjusets'),
    (' michael', ' majkel'),
    ('microsoft', 'majkrosoft'),
    ('raymond', 'rejmond'),
    (' mike', ' majk'),
    (' miles', ' majls'),
    (' mount', ' małnt'),
    (' movie', ' muwi'),
    (' murphy', ' merfi'),
    (' nicol', ' nikol'),
    (' owen', ' ołen'),
    (' pierre', ' pier'),
    ('requiem', 'rekfiem'),
    (' richard', ' riczard'),
    (' schubert', ' szubert'),
    (' schwartz', ' szwarc'),
    (' scotland', ' skotland'),
    (' schroeder', ' szreder'),
    (' schlesinger', ' szlezinger'),
    (' scott', ' skot'),
    ('siergiej', 'sjergiej'),
    (' skype', ' skajp'),
    (' spice', ' spajs'),
    (' steal ', ' stil '),
    (' stephen', ' stiwen'),
    (' steven', ' stiwen'),
    (' stuart', ' stju_art'),
    (' tequil', ' tekil'),
    (' tracy', ' trejs_i'),
    (' train', ' trejn'),
    (' true ', ' tru '),
    (' victor', ' wiktor'),
    (' wells', ' łels'),
    (' william', ' łyliam'),
    (' wilson', ' łylson'),
    (' world', ' łerld'),
    ('sawyer', ' sojer'),
    ('manhattan', 'manhatan'),
    ('morningside', 'morningsajd'),
    ('highway', 'hajłej'),
    (' frau ', ' frał '),
    ('deutsche', 'dojcze'),
    ('strauss', 'sztrałs'),
    ('rozenthal', 'rozental'),
    ('louis', 'lułis'),
    ('hollywood', 'holiłud'),
    (' madame ', ' madam '),
    ('steel', 'stil'),
    ('dashwood', 'daszłud'),
    ('middleton', 'midlton'),
    ('street', 'strit'),
    ('jennings', 'dżenings'),
    ('temple ', 'templ'),
    ('t-shirt', 'tiszert'),
    ('smith', 'smif'),
    ('thatcher', 'taczer'),
    ('cardiff', 'kardif'),
    ('cardiff', 'kardif'),
    (' hill ', ' hil '),
    ('douglas', 'daglas'),
    ('lunch', 'lancz'),
    ('york', 'jork'),
    ('crowmont', 'krołmont'),
    (' brian', 'brajan'),
    (' withmore', 'łifmor'),
    (' whitmore', 'łitmor'),
    ('whitemore', 'łajtmor'),
    (' cowell', 'kołel'),
    ('scrooge', 'skrudż'),
    ('marley', 'marlej'),
    (' pepsi', ' peps_i'),
    ('cornhill', 'korn_hill'),
    (' camden', ' kamden'),
    (' town ', ' tałn '),
    ('debbie', 'debi'),
    (' cowboy', ' kałboj'),
    (' pool', ' pul'),
    (' chi ', ' czi '),
    ('ernshow', 'ernszoł'),
    ('speech', 'spicz'),
    ('bright', 'brajt'),
    ('white', 'łajt'),
    ('francesc', 'franczesk'),
    ('skellige', 'skelige'),
    ('percival', 'pers_iwal'),
    ('chivay', 'cziwaj'),
    ('djinn', 'dżin'),
    ('incognito', 'inkognito'),
    ('vilgefortz', 'vilgeforc'),
    (' ciri', ' c_iri'),
    (' metz ', ' mec '),
    (' dick', ' dik'),
    ('pudding', 'pading'),
    ('thyssen', 'fyssen'),
    ('lorenzo', 'lorenco'),
    ('schiller', 'sziler'),
    ('caesar', 'cezar'),
    ('hemicrania', 'hemikrania'),
    ('procurator', 'prokurator'),
    ('aurillac', 'aurillak'),
    ('stratonic', 'stratonik'),
    ('gaziet', 'gasjet'),
    ('ad acta', 'ad akta'),
    ('de facto', 'de fakto'),
    ('goethe', 'gete'),
    ('ambrosij', 'ambroz_ij'),
    ('colosseum', 'koloseum'),
    ('au naturel', 'o natjurel'),
    ('shakespeare', 'szekspir'),
    (' menu ', ' meni '),
    (' kaer ', ' ker '),
    (' morhen', ' moren'),
    (' tizian', ' ticjan'),
    (' toussaint', ' tusą'),
    ('vsky', 'wski'),
    ('thank you', 'fenk ju'),
    (' nein ', ' najn '),
    ('qun', 'kun'),
    ('lays', 'lejs'),
    (' chip', ' czip'),
    (' voodoo', ' wudu'),
    (' runner', ' raner'),
    (' runner', ' raner'),
    ('afterlife', 'afterlajf'),
    (' night', ' najt'),
    (' city', ' s_iti'),
    (' wasil', ' was_il'),
    (' jazz', ' dżez'),
    (' de volaille', ' de wolajl'),
    ('kalkstein', 'kalksztajn'),
    ('goldstein', 'goldsztajn'),
    ("scoia'tael", 'skoja tel'), 
    ("abigail", 'abigejl'),
    ('beatles', 'bitels'),
    ('w ogóle', 'wogóle'),
    ('starkhaven', 'starkhejwen'),
    (' north', ' norf'),
    (' oak', ' ołk'),
    ('rainbow', 'rejnboł'),
    ('underground', 'andergrand'),
    ('hand ', 'hend '),
    ('hobby', 'hobbi'),
    ('winston', 'łinston'),
    ('status quo', 'status kwo'),

    # (' ', ''),
    ('m.in.', 'między innymi '),
    (' etc.', 'et_cetera '),
    (' pt.', ' pod tytułem '),
    (' vhs ', ' fał ha es '),
    (' nr ', ' numer '),
    
    # (' ', ''),
    ('é', 'e'),
    ('ü', 'y'),
    ('ö', 'y'),
    ('ä', 'e'),
    ('x', 'ks'),
    ('sh', 'sz'),
    ('zż', 'z-ż'),
    ('rż', 'r-ż'),
    ('zsz', 'z_sz'),
    ('ższ', 'ż_sz'),
    ('ww', 'w\'w'),
    ('nn', 'n\'n'),
    ('ss', 's\'s'),
    ('tc', 't\'c'),
    ('dd', 'd\'d'),
    ('pp', 'p\'p'),
    ('tt', 't\'t'),
    ('zz', 'z\'z'),
]
    
def gen_rule(rule_tuple: Tuple[str, str]) -> Tuple[re.Pattern, str]:
    from_rule, to_rule = rule_tuple
    prefix, suffix = False, False
    
    from_rule = from_rule.replace('.', '\\.')

    if from_rule[0] == ' ':
        prefix = True
    
    if from_rule[-1] == ' ':
        suffix = True
    
    _prefix = '([ \,\.\;\!\:\?]|^)' if prefix else ''
    _suffix = '([ \,\.\;\!\:\?]|$)' if suffix else ''

    _from_rule = (f'{_prefix}{from_rule.strip()}{_suffix}')

    _to_prefix = '\g<1>' if _prefix != '' else ''
    _to_suffix = '\g<1>' if _prefix == '' and _suffix != '' else '\g<2>' if _suffix != '' else ''

    _to_rule = (f'{_to_prefix}{to_rule.strip()}{_to_suffix}')

    return re.compile(_from_rule), _to_rule

rules = [*map(gen_rule, rules_def)]

prelower_rules = [
    (' GLKiH', ' Gie eL Ka i Ha'),
    (' GSzG', 'Gie es zet Gie'),
    (' SWD', ' eS Wu De'),
    (' DNA', ' De En A'),
    (' USA', ' U eS A'),
    ('Red Mount', 'Red Małnt'),
    (' Zin', ' Z_in'),
    ('Will', 'Łil'),
    ('Andy', 'Endi'),
    ('Betty', 'Beti'),
    (' Vincent', ' Winsent'),
    ('Trump', 'Tramp'),
    ('Fieldstone', 'Fildston'),
    ('West Side', 'Łest Sajd'),
    ('Hudson', 'Hadson'),
    ('Hofmeister', 'Hofmajster'),
    ('Lucy ', 'Lus_i '),
    ('Lady ', 'Lejdi '),
    ('Marianne ', 'Marian '),
    ('Cleveland', 'Kliwlend'),
    ('Willoughby', 'Łilołbi'),
    ('Jake ', 'Dżejk '),
    ('Jessie', 'Dżes_i'),
    ('Andrews', 'Endrjus'),
    ('Carmen', 'Karmen'),
    ('Doug ', 'Dag '),
    ('Craig', 'Krejg'),
    ('Sophie', 'Sofi'),
    ('Carlit', 'Karlit'),
    (' Misty ', ' Misti '),
    ('Joe', 'Dżo'),
    ('Kate ', 'Kejt '),
    ('Oate', 'Ołen'),
    ('Maisy', 'Mejz_i'),
    (' Simon', ' Sajmon'),
    (' Boyce', ' Bojs'),
    (' Yennefer', ' Jenefer'),
    (' Neyd', ' Nejd'),
    (' Chaplin', ' Czaplyn'),
    (' SOS ', ' es o es '),
    (' Col', ' Kol'),
    (' SS ', ' eS eS '),
    (' Schtreck ', ' Sztrek '),
    ('Mary Poppins', 'Mery Popiins'),
    ('Mein Kampf', 'Majn Kampf'),
    ('La Manche', 'La-Mansz'),
    ('Führer', 'Firer'),
    ('Ahnenerbe', 'Anen_Erbe'),
    ('MacMillan', 'Mak_Milan'),
    (' Mac ', ' Mak '),
    (' Buck', ' Bak'),
    (' Randy', ' Randi'),
    (' Nellie', ' Neli'),
    (' Cooper', ' Kuper'),
    (' Jack ', ' Dżak '),
    (' Randy', ' Randij'),
    (' Michigan', ' Miszigen'),
    (' Bobby', ' Bobi'),
    (' Buster', ' Baster'),
    (' Keaton', ' Kiton'),
    (' Goebbel', ' Gebel'),
    (' Heil', ' Hajl'),
    (' Riefenstahl', ' Rifensztal'),
    (' Einstein', ' Ajnsztajn'),
    (' Walt', ' Łolt'),
    (' Disney', ' Di_znyj'),
    ('Dmitry', 'Dmitri'),
    ('Copperfield', 'Koperfild'),
    ('We all live in our yellow submarine, Yellow submarine, yellow submarine', 'Łi ol liw in ałer jeloł submarin, jeloł submarin, jeloł submarin'),
    ('And our friends are all aboard, Many more of them live next door', 'end ałer frends ar ol e-bord, men_i mor of wem liw next dor'),
    ('And the band begins to play', 'end we band begins tu plej'),
    ('My God… White skin, this is really white skin', 'maj god… łajt skin, dis is rili łajt skin'),
    ('Не asked, where did you come from', 'hi askt, łer did ju kam from'),
    ("A white face! After so many years! Please, help me! I 'm a citizen of the United States", 'e łajt fejs! after soł men_i jers! pliz, help mi! ajm e s_itizen of we junajted stejts'),
    ("We were shot down twenty years ago… In the desert.", 'łi łer szot dałn tłenti jers egoł... in we dezert.'),
    ("Pilots were killed, and I was imprisoned by these savages", 'pajlots łer kilt, end aj łos imprizont baj diz sawydżys'),
    ("You are Russian, right", 'ju ar raszjan, rajt'),
    ("I 'm lieutenant Jenny Tahoma", 'ajm lewtenant dżen_i tahoma'),
    ("Are you from military", 'ar ju from militari'),
    ("Did you come here with a group", 'did ju kam hir łyf e grup'),
    ("How many people are with you", 'hał men_i pipyl ar łyf ju'),
    ("It will kill you", 'it łil kil ju'),
    ("Bitte geben Sie mir ein Schluck", 'bite gejben z_i mir ajn szluk'),
    ("Wasser", 'Wassaa'),
]

rules2 = [*map(gen_rule, prelower_rules)]

def shmart_cleaner(text: str) -> str:
    if not len(text):
        return '___'
    text = normalize_numbers(text)
    for _, (rule_from, rule_to) in enumerate(rules2):
        text = rule_from.sub(rule_to, text)

    text = text[1:] if text[0] == '-' else text
    text = text.lower()

    for _, (rule_from, rule_to) in enumerate(rules):
        text = rule_from.sub(rule_to, text)
    
    if text.strip()[-1] not in ['.', ',', '!', '-', '?', ';', ':']:
        text += ','
    
    return text

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--text', type=str, help='text to clean', default='Podziemny krąg.')
    args = parser.parse_args()

    print(shmart_cleaner(args.text))


import copy
import hashlib
import json
import librosa
import numpy as np
import os
import pathlib
import requests
import shutil
import soundfile as sf
import threading
import torch
import torch.nn as nn
import torch.nn.functional as F

from tqdm import tqdm

from shttst.audio.utils import convert_to_mono_channel, save_audio
from zipfile import ZipFile

CHECKPOINT_CACHE_PATH = os.path.expanduser('~/.cache/shttst/ultimate_vocal_remover/HP2-4BAND-3090_4band_2.pth')
ZIP_PATH = os.path.expanduser('~/.cache/shttst/ultimate_vocal_remover/uvr.zip')

def get_checkpoint():
    if not os.path.exists(CHECKPOINT_CACHE_PATH):
        os.makedirs(os.path.dirname(CHECKPOINT_CACHE_PATH), exist_ok=True)
        resp = requests.get('https://github.com/Anjok07/ultimatevocalremovergui/releases/download/5.0.2/v5_July_2021_5_Models.zip')
        with open(ZIP_PATH, 'w+b') as fs:
            fs.write(resp.content)
        with ZipFile(ZIP_PATH, 'r') as zipped_vr:
            zipped_vr.extract('models/HP2-4BAND-3090_4band_2.pth', os.path.dirname(CHECKPOINT_CACHE_PATH))
            
        shutil.move(os.path.join(os.path.dirname(CHECKPOINT_CACHE_PATH), 'models/HP2-4BAND-3090_4band_2.pth'), CHECKPOINT_CACHE_PATH)
        shutil.rmtree(os.path.join(os.path.dirname(CHECKPOINT_CACHE_PATH), 'models'), ignore_errors=False, onerror=None)
        os.remove(ZIP_PATH)

    return CHECKPOINT_CACHE_PATH

# Taken from: https://github.com/Anjok07/ultimatevocalremovergui/tree/v5-beta-cml

default_param = {}
default_param['bins'] = 768
default_param['unstable_bins'] = 7 # training only
default_param['reduction_bins'] = 668 # training only
default_param['sr'] = 44100
default_param['pre_filter_start'] = 740
default_param['pre_filter_stop'] = 768
default_param['band'] = {}


default_param['band'][1] = {
    'sr': 11025,
    'hl': 128,
    'n_fft': 1024,
    'crop_start': 0,
    'crop_stop': 186,
    'lpf_start': 37,
    'lpf_stop': 73,
    'res_type': 'polyphase'
}

default_param['band'][2] = {
    'sr': 11025,
    'hl': 128,
    'n_fft': 512,
    'crop_start': 4,
    'crop_stop': 185,
    "hpf_start": 36,
    "hpf_stop": 18,
    'lpf_start': 93,
    'lpf_stop': 185,
    'res_type': 'polyphase'
}

default_param['band'][3] = {
    'sr': 22050,
    'hl': 256,
    'n_fft': 512,
    'crop_start': 46,
    'crop_stop': 186,
    "hpf_start": 93,
    "hpf_stop": 46,
    'lpf_start': 164,
    'lpf_stop': 186,
    'res_type': 'polyphase'
}

default_param['band'][4] = {
    'sr': 44100,
    'hl': 512,
    'n_fft': 768,
    'crop_start': 121,
    'crop_stop': 382,
    "hpf_start": 138,
    "hpf_stop": 123,
    'res_type': 'sinc_medium'
}

def int_keys(d):
    r = {}
    for k, v in d:
        if k.isdigit():
            k = int(k)
        r[k] = v
    return r

class ModelParameters(object):
    def __init__(self, config_path=''):
        if '.pth' == pathlib.Path(config_path).suffix:
            import zipfile
            
            with zipfile.ZipFile(config_path, 'r') as zip:
                self.param = json.loads(zip.read('param.json'), object_pairs_hook=int_keys)
        elif '.json' == pathlib.Path(config_path).suffix:
            with open(config_path, 'r') as f:
                self.param = json.loads(f.read(), object_pairs_hook=int_keys)
        else:
            self.param = default_param
            
        for k in ['mid_side', 'mid_side_b', 'mid_side_b2', 'stereo_w', 'stereo_n', 'reverse']:
            if not k in self.param:
                self.param[k] = False

def crop_center(h1, h2):
    h1_shape = h1.size()
    h2_shape = h2.size()

    if h1_shape[3] == h2_shape[3]:
        return h1
    elif h1_shape[3] < h2_shape[3]:
        raise ValueError('h1_shape[3] must be greater than h2_shape[3]')

    # s_freq = (h2_shape[2] - h1_shape[2]) // 2
    # e_freq = s_freq + h1_shape[2]
    s_time = (h1_shape[3] - h2_shape[3]) // 2
    e_time = s_time + h2_shape[3]
    h1 = h1[:, :, :, s_time:e_time]

    return h1

   
def wave_to_spectrogram(wave, hop_length, n_fft, mp, multithreading):      
    wave_left = np.asfortranarray(wave[0])
    wave_right = np.asfortranarray(wave[1])

    if multithreading:
        def run_thread(**kwargs):
            global spec_left_mt
            spec_left_mt = librosa.stft(**kwargs)

        thread = threading.Thread(target=run_thread, kwargs={'y': wave_left, 'n_fft': n_fft, 'hop_length': hop_length})
        thread.start()
        spec_right = librosa.stft(wave_right, n_fft=n_fft, hop_length=hop_length)
        thread.join()
        spec = np.asfortranarray([spec_left_mt, spec_right])
    else:
        spec_left = librosa.stft(wave_left, n_fft, hop_length=hop_length)
        spec_right = librosa.stft(wave_right, n_fft, hop_length=hop_length)
        spec = np.asfortranarray([spec_left, spec_right])

    return spec
    

def convert_channels(spec, mp, band):
    cc = mp.param['band'][band].get('convert_channels')

    if mp.param['reverse']:
        spec_left = np.flip(spec[0])
        spec_right = np.flip(spec[1])
    elif mp.param['mid_side_b'] or 'mid_side_b' == cc:
        spec_left = np.add(spec[0], spec[1] * .5)
        spec_right = np.subtract(spec[1], spec[0] * .5)
    elif mp.param['mid_side_b2'] or 'mid_side_b2' == cc:
        spec_left = np.add(spec[1], spec[0] * .5)
        spec_right = np.subtract(spec[0], spec[1] * .5)
    elif 'mid_side_c' == cc:
        spec_left = np.add(spec[0], spec[1] * .25)
        spec_right = np.subtract(spec[1], spec[0] * .25)
    elif mp.param['mid_side'] or 'mid_side' == cc:
        spec_left = np.add(spec[0], spec[1]) / 2
        spec_right = np.subtract(spec[0], spec[1])
    elif mp.param['stereo_n']:
        spec_left = np.add(spec[0], spec[1] * .25) / 0.9375
        spec_right = np.add(spec[1], spec[0] * .25) / 0.9375
    else:
        return spec
        
    return np.asfortranarray([spec_left, spec_right])
    

def combine_spectrograms(specs, mp):
    l = min([specs[i].shape[2] for i in specs])    
    spec_c = np.zeros(shape=(2, mp.param['bins'] + 1, l), dtype=np.complex64)
    offset = 0
    bands_n = len(mp.param['band'])
    
    for d in range(1, bands_n + 1):
        h = mp.param['band'][d]['crop_stop'] - mp.param['band'][d]['crop_start']
        s = specs[d][:, mp.param['band'][d]['crop_start']:mp.param['band'][d]['crop_stop'], :l]
        #if 'flip' in mp.param['band'][d]:
        #    s = np.flip(s, 1)
        spec_c[:, offset:offset+h, :l] = s
        offset += h
        
    if offset > mp.param['bins']:
        raise ValueError('Too much bins')
        
    if mp.param['pre_filter_start'] > 0:  
        #if bands_n == 1:
        spec_c *= get_lp_filter_mask(spec_c.shape[1], mp.param['pre_filter_start'], mp.param['pre_filter_stop'])
        '''else:
            gp = 1        
            for b in range(mp.param['pre_filter_start'] + 1, mp.param['pre_filter_stop']):
                g = math.pow(10, -(b - mp.param['pre_filter_start']) * (3.5 - gp) / 20.0)
                gp = g
                spec_c[:, b, :] *= g
        '''
                
    return np.asfortranarray(spec_c)


def spectrogram_to_image(spec, mode='magnitude'):
    if mode == 'magnitude':
        if np.iscomplexobj(spec):
            y = np.abs(spec)
        else:
            y = spec
        y = np.log10(y ** 2 + 1e-8)
    elif mode == 'phase':
        if np.iscomplexobj(spec):
            y = np.angle(spec)
        else:
            y = spec

    y -= y.min()
    y *= 255 / y.max()
    img = np.uint8(y)

    if y.ndim == 3:
        img = img.transpose(1, 2, 0)
        img = np.concatenate([
            np.max(img, axis=2, keepdims=True), img
        ], axis=2)

    return img


def reduce_vocal_aggressively(X, y, softmask):
    v = X - y
    y_mag_tmp = np.abs(y)
    v_mag_tmp = np.abs(v)

    v_mask = v_mag_tmp > y_mag_tmp
    y_mag = np.clip(y_mag_tmp - v_mag_tmp * v_mask * softmask, 0, np.inf)

    return y_mag * np.exp(1.j * np.angle(y))


def mask_silence(mag, ref, thres=0.2, min_range=64, fade_size=32):
    if min_range < fade_size * 2:
        raise ValueError('min_range must be >= fade_area * 2')

    mag = mag.copy()

    idx = np.where(ref.mean(axis=(0, 1)) < thres)[0]
    starts = np.insert(idx[np.where(np.diff(idx) != 1)[0] + 1], 0, idx[0])
    ends = np.append(idx[np.where(np.diff(idx) != 1)[0]], idx[-1])
    uninformative = np.where(ends - starts > min_range)[0]
    if len(uninformative) > 0:
        starts = starts[uninformative]
        ends = ends[uninformative]
        old_e = None
        for s, e in zip(starts, ends):
            if old_e is not None and s - old_e < fade_size:
                s = old_e - fade_size * 2

            if s != 0:
                weight = np.linspace(0, 1, fade_size)
                mag[:, :, s:s + fade_size] += weight * ref[:, :, s:s + fade_size]
            else:
                s -= fade_size

            if e != mag.shape[2]:
                weight = np.linspace(1, 0, fade_size)
                mag[:, :, e - fade_size:e] += weight * ref[:, :, e - fade_size:e]
            else:
                e += fade_size

            mag[:, :, s + fade_size:e - fade_size] += ref[:, :, s + fade_size:e - fade_size]
            old_e = e

    return mag
    

def trim_specs(a, b):
    l = min([a.shape[2], b.shape[2]])  
    
    return a[:,:,:l], b[:,:,:l]


def cache_or_load(mix_path, inst_path, mp):
    mix_basename = os.path.splitext(os.path.basename(mix_path))[0]
    inst_basename = os.path.splitext(os.path.basename(inst_path))[0]
    
    # the cache will be common for some model types
    mpp2 = copy.deepcopy(mp.param)
    mpp2.update(dict.fromkeys(['mid_side', 'mid_side_b', 'mid_side_b2', 'reverse'], False))
    
    for d in mpp2['band']:
        mpp2['band'][d]['convert_channels'] = ''

    cache_dir = 'mp{}'.format(hashlib.sha1(json.dumps(mpp2, sort_keys=True).encode('utf-8')).hexdigest())
    mix_cache_dir = os.path.join('cache', cache_dir)
    inst_cache_dir = os.path.join('cache', cache_dir)

    os.makedirs(mix_cache_dir, exist_ok=True)
    os.makedirs(inst_cache_dir, exist_ok=True)

    mix_cache_path = os.path.join(mix_cache_dir, mix_basename + '.npy')
    inst_cache_path = os.path.join(inst_cache_dir, inst_basename + '.npy')

    if os.path.exists(mix_cache_path) and os.path.exists(inst_cache_path):
        X_spec_m = np.load(mix_cache_path)
        y_spec_m = np.load(inst_cache_path)
    else:
        '''
        X_wave, y_wave, X_spec_s, y_spec_s = {}, {}, {}, {}
         
        for d in range(len(mp.param['band']), 0, -1):            
            bp = mp.param['band'][d]
                    
            if d == len(mp.param['band']): # high-end band
                X_wave[d], _ = librosa.load(
                    mix_path, bp['sr'], False, dtype=np.float32, res_type=bp['res_type'])
                y_wave[d], _ = librosa.load(
                    inst_path, bp['sr'], False, dtype=np.float32, res_type=bp['res_type'])
            else: # lower bands
                X_wave[d] = librosa.resample(X_wave[d+1], mp.param['band'][d+1]['sr'], bp['sr'], res_type=bp['res_type'])
                y_wave[d] = librosa.resample(y_wave[d+1], mp.param['band'][d+1]['sr'], bp['sr'], res_type=bp['res_type'])
            
            X_wave[d], y_wave[d] = align_wave_head_and_tail(X_wave[d], y_wave[d])
            
            X_spec_s[d] = wave_to_spectrogram(X_wave[d], bp['hl'], bp['n_fft'], mp, False)
            y_spec_s[d] = wave_to_spectrogram(y_wave[d], bp['hl'], bp['n_fft'], mp, False)
            
        del X_wave, y_wave
                 
        X_spec_m = combine_spectrograms(X_spec_s, mp)
        y_spec_m = combine_spectrograms(y_spec_s, mp)
        '''
        
        X_spec_m = spec_from_file(mix_path, mp)
        y_spec_m = spec_from_file(inst_path, mp)
        
        X_spec_m, y_spec_m = trim_specs(X_spec_m, y_spec_m)
        
        
        if X_spec_m.shape != y_spec_m.shape:
            raise ValueError('The combined spectrograms are different: ' + mix_path)

        _, ext = os.path.splitext(mix_path)

        np.save(mix_cache_path, X_spec_m)
        np.save(inst_cache_path, y_spec_m)

    return X_spec_m, y_spec_m

    
def spectrogram_to_wave(spec, hop_length, mp, band, multithreading):
    import threading

    spec_left = np.asfortranarray(spec[0])
    spec_right = np.asfortranarray(spec[1])
    cc = mp.param['band'][band].get('convert_channels')
    
    if multithreading:
        def run_thread(**kwargs):
            global wave_left_mt
            wave_left_mt = librosa.istft(**kwargs)
            
        thread = threading.Thread(target=run_thread, kwargs={'stft_matrix': spec_left, 'hop_length': hop_length})
        thread.start()
        wave_right = librosa.istft(spec_right, hop_length=hop_length)
        thread.join()   
        wave_left = wave_left_mt
    else:
        wave_left = librosa.istft(spec_left, hop_length=hop_length)
        wave_right = librosa.istft(spec_right, hop_length=hop_length)
    
    if mp.param['reverse']:
        return np.asfortranarray([np.flip(wave_left), np.flip(wave_right)])
    elif mp.param['mid_side_b'] or 'mid_side_b' == cc:
        return np.asfortranarray([np.subtract(wave_left / 1.25, .4 * wave_right), np.add(wave_right / 1.25, .4 * wave_left)])       
    elif mp.param['mid_side_b2'] or 'mid_side_b2' == cc:
        return np.asfortranarray([np.add(wave_right / 1.25, .4 * wave_left), np.subtract(wave_left / 1.25, .4 * wave_right)])
    elif 'mid_side_c' == cc:
        return np.asfortranarray([np.subtract(wave_left / 1.0625, wave_right / 4.25), np.add(wave_right / 1.0625, wave_left / 4.25)])    
    elif mp.param['mid_side'] or 'mid_side' == cc:
        return np.asfortranarray([np.add(wave_left, wave_right / 2), np.subtract(wave_left, wave_right / 2)])
    elif mp.param['stereo_n']:
        return np.asfortranarray([np.subtract(wave_left, wave_right * .25), np.subtract(wave_right, wave_left * .25)])
    else:
        return np.asfortranarray([wave_left, wave_right])

    
def cmb_spectrogram_to_wave(spec_m, mp, extra_bins_h=None, extra_bins=None):
    wave_band = {}
    bands_n = len(mp.param['band'])    
    offset = 0
   
    for d in range(1, bands_n + 1):
        bp = mp.param['band'][d]
        spec_s = np.zeros(shape=(2, bp['n_fft'] // 2 + 1, spec_m.shape[2]), dtype=complex)
        h = bp['crop_stop'] - bp['crop_start']
        #if 'flip' in mp.param['band'][d]:
        #    spec_s[:, bp['crop_start']:bp['crop_stop'], :] = np.flip(spec_m[:, offset:offset+h, :], 1)
        #else:
        spec_s[:, bp['crop_start']:bp['crop_stop'], :] = spec_m[:, offset:offset+h, :]
        
        offset += h
        if d == bands_n: # high-end
            if extra_bins_h:
                max_bin = bp['n_fft'] // 2
                spec_s[:, max_bin-extra_bins_h:max_bin, :] = extra_bins[:, :extra_bins_h, :]
            if bp['hpf_start'] > 0:
                spec_s *= get_hp_filter_mask(spec_s.shape[1], bp['hpf_start'], bp['hpf_stop'] - 1)
            if bands_n == 1:
                wave = spectrogram_to_wave(spec_s, bp['hl'], mp, d, False)
            else:
                wave = np.add(wave, spectrogram_to_wave(spec_s, bp['hl'], mp, d, False))
        else:
            sr = mp.param['band'][d+1]['sr']
            if d == 1: # low-end
                spec_s *= get_lp_filter_mask(spec_s.shape[1], bp['lpf_start'], bp['lpf_stop'])
                wave = librosa.resample(spectrogram_to_wave(spec_s, bp['hl'], mp, d, False), bp['sr'], sr, res_type="sinc_fastest")
            else: # mid
                spec_s *= get_hp_filter_mask(spec_s.shape[1], bp['hpf_start'], bp['hpf_stop'] - 1)
                spec_s *= get_lp_filter_mask(spec_s.shape[1], bp['lpf_start'], bp['lpf_stop'])
                wave2 = np.add(wave, spectrogram_to_wave(spec_s, bp['hl'], mp, d, False))
                wave = librosa.resample(wave2, bp['sr'], sr, res_type="sinc_fastest")
        
    return wave.T


def cmb_spectrogram_to_wave_ffmpeg(spec_m, mp, tmp_basename, extra_bins_h=None, extra_bins=None):
    import subprocess
    os.makedirs('tmp', exist_ok=True)

    bands_n = len(mp.param['band'])    
    offset = 0
    ffmprc = {}

    for d in range(1, bands_n + 1):
        bp = mp.param['band'][d]
        spec_s = np.zeros(shape=(2, bp['n_fft'] // 2 + 1, spec_m.shape[2]), dtype=complex)
        h = bp['crop_stop'] - bp['crop_start']
        spec_s[:, bp['crop_start']:bp['crop_stop'], :] = spec_m[:, offset:offset+h, :]
        tmp_wav = os.path.join('tmp', '{}_cstw_b{}_sr{}'.format(tmp_basename, d, str(bp['sr']) + '.wav'))
        tmp_wav2 = os.path.join('tmp', '{}_cstw_b{}_sr{}'.format(tmp_basename, d, str(mp.param['sr']) + '.wav'))
        
        offset += h
        if d == bands_n: # high-end
            if extra_bins_h:
                max_bin = bp['n_fft'] // 2
                spec_s[:, max_bin-extra_bins_h:max_bin, :] = extra_bins[:, :extra_bins_h, :]
            if bp['hpf_start'] > 0:
                spec_s *= get_hp_filter_mask(spec_s.shape[1], bp['hpf_start'], bp['hpf_stop'] - 1)
            if bands_n == 1:
                wave = spectrogram_to_wave(spec_s, bp['hl'], mp, d, True)
            else:
                wave = spectrogram_to_wave(spec_s, bp['hl'], mp, d, True)
        else:
            if d == 1: # low-end
                spec_s *= get_lp_filter_mask(spec_s.shape[1], bp['lpf_start'], bp['lpf_stop'])
            else: # mid
                spec_s *= get_hp_filter_mask(spec_s.shape[1], bp['hpf_start'], bp['hpf_stop'] - 1)
                spec_s *= get_lp_filter_mask(spec_s.shape[1], bp['lpf_start'], bp['lpf_stop'])

            sf.write(tmp_wav, spectrogram_to_wave(spec_s, bp['hl'], mp, d, True).T, bp['sr'])
            ffmprc[d] = subprocess.Popen(['ffmpeg', '-hide_banner', '-loglevel', 'panic', '-y', '-i', tmp_wav, '-ar', str(mp.param['sr']), '-ac', '2', '-c:a', 'pcm_s16le', tmp_wav2])

    for s in ffmprc:
        ffmprc[s].communicate()
        
    for d in range(bands_n - 1, 0, -1):
        os.remove(os.path.join('tmp', f'{tmp_basename}_cstw_b{d}_sr' + str(mp.param['band'][d]['sr']) + '.wav'))
        tmp_wav2 = os.path.join('tmp', f'{tmp_basename}_cstw_b{d}_sr' + str(mp.param['sr']) + '.wav')
        wave2, _ = librosa.load(tmp_wav2, sr=mp.param['sr'], mono=False, dtype=np.float32, res_type="sinc_fastest")
        os.remove(tmp_wav2)
        wave = np.add(wave, wave2)

    return wave.T

'''
def fft_lp_filter(spec, bin_start, bin_stop):
    g = 1.0
    for b in range(bin_start, bin_stop):
        g -= 1 / (bin_stop - bin_start)
        spec[:, b, :] = g * spec[:, b, :]
        
    spec[:, bin_stop:, :] *= 0
    return spec
def fft_hp_filter(spec, bin_start, bin_stop):
    g = 1.0
    for b in range(bin_start, bin_stop, -1):
        g -= 1 / (bin_start - bin_stop)
        spec[:, b, :] = g * spec[:, b, :]
    
    spec[:, 0:bin_stop+1, :] *= 0
    return spec
'''
    
def get_lp_filter_mask(bins_n, bin_start, bin_stop):
    mask = np.concatenate([
        np.ones((bin_start - 1, 1)),
        np.linspace(1, 0, bin_stop - bin_start + 1)[:, None],
        np.zeros((bins_n - bin_stop, 1))
    ], axis=0)

    return mask
    
    
def get_hp_filter_mask(bins_n, bin_start, bin_stop):
    mask = np.concatenate([
        np.zeros((bin_stop + 1, 1)),
        np.linspace(0, 1, 1 + bin_start - bin_stop)[:, None],
        np.ones((bins_n - bin_start - 2, 1))
    ], axis=0)

    return mask


def mirroring(a, spec_m, input_high_end, mp):
    if 'mirroring' == a:
        mirror = np.flip(np.abs(spec_m[:, mp.param['pre_filter_start']-10-input_high_end.shape[1]:mp.param['pre_filter_start']-10, :]), 1)
        mirror = mirror * np.exp(1.j * np.angle(input_high_end))
        
        return np.where(np.abs(input_high_end) <= np.abs(mirror), input_high_end, mirror)
        
    if 'mirroring2' == a:
        mirror = np.flip(np.abs(spec_m[:, mp.param['pre_filter_start']-10-input_high_end.shape[1]:mp.param['pre_filter_start']-10, :]), 1)
        mi = np.multiply(mirror, input_high_end * 1.7)
        
        return np.where(np.abs(input_high_end) <= np.abs(mi), input_high_end, mi)
        
        
def adjust_aggr(mask, params):
    aggr = params.get('aggr_value', 0.0)

    if aggr != 0:
        if params.get('is_vocal_model'):
            aggr = 1 - aggr
    
        aggr_l = aggr_r = aggr
    
        if params['aggr_correction'] is not None:
            aggr_l += params['aggr_correction']['left']
            aggr_r += params['aggr_correction']['right']
        
        mask[:, 0, :params['aggr_split_bin']] = torch.pow(mask[:, 0, :params['aggr_split_bin']], 1 + aggr_l / 3)
        mask[:, 0, params['aggr_split_bin']:] = torch.pow(mask[:, 0, params['aggr_split_bin']:], 1 + aggr_l)
        
        mask[:, 1, :params['aggr_split_bin']] = torch.pow(mask[:, 1, :params['aggr_split_bin']], 1 + aggr_r / 3)
        mask[:, 1, params['aggr_split_bin']:] = torch.pow(mask[:, 1, params['aggr_split_bin']:], 1 + aggr_r)

    return mask
        

def ensembling(a, specs, sr=44100):   
    for i in range(1, len(specs)):
        if i == 1:
            spec = specs[0]

        ln = min([spec.shape[2], specs[i].shape[2]])
        spec = spec[:,:,:ln]
        specs[i] = specs[i][:,:,:ln]
        freq_to_bin = 2 * spec.shape[1] / sr

        if 'min_mag' == a:
            spec = np.where(np.abs(specs[i]) <= np.abs(spec), specs[i], spec)
        if 'max_mag' == a:
            spec = np.where(np.abs(specs[i]) >= np.abs(spec), specs[i], spec)         
        if 'mul' == a:
            s1 = specs[i] * spec
            s2 = .5 * (specs[i] + spec)
            spec = np.divide(s1, s2, out=np.zeros_like(s1), where=s2!=0)
        if 'crossover' == a:
            bs = int(500 * freq_to_bin)
            be = int(14000 * freq_to_bin)
            spec = specs[i] * get_lp_filter_mask(spec.shape[1], bs, be) + spec * get_hp_filter_mask(spec.shape[1], be, bs)
        if 'min_mag_co' == a:
            specs[i] += specs[i] * get_hp_filter_mask(spec.shape[1], int(14000 * freq_to_bin), int(4000 * freq_to_bin))
            spec = np.where(np.abs(specs[i]) <= np.abs(spec), specs[i], spec)      

    return spec


def spec_from_file(filename, mp):
    wave, spec = {}, {}
    
    for d in range(len(mp.param['band']), 0, -1):          
        bp = mp.param['band'][d]            
        
        if d == len(mp.param['band']): # high-end band                
            wave, _ = librosa.load(
                filename, bp['sr'], False, dtype=np.float32, res_type=bp['res_type'])
            
            if len(wave.shape) == 1: # mono to stereo
                wave = np.array([wave, wave])
        else: # lower bands
            wave = librosa.resample(wave, mp.param['band'][d+1]['sr'], bp['sr'], res_type=bp['res_type'])
                   
        spec[d] = wave_to_spectrogram(wave, bp['hl'], bp['n_fft'], mp, False)
        spec[d] = convert_channels(spec[d], mp, d)

    return combine_spectrograms(spec, mp)

class Conv2DBNActiv(nn.Module):

    def __init__(self, nin, nout, ksize=3, stride=1, pad=1, dilation=1, activ=nn.ReLU):
        super(Conv2DBNActiv, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(
                nin, nout,
                kernel_size=ksize,
                stride=stride,
                padding=pad,
                dilation=dilation,
                bias=False),
            nn.BatchNorm2d(nout),
            activ()
        )

    def __call__(self, x):
        return self.conv(x)


class SeperableConv2DBNActiv(nn.Module):

    def __init__(self, nin, nout, ksize=3, stride=1, pad=1, dilation=1, activ=nn.ReLU):
        super(SeperableConv2DBNActiv, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(
                nin, nin,
                kernel_size=ksize,
                stride=stride,
                padding=pad,
                dilation=dilation,
                groups=nin,
                bias=False),
            nn.Conv2d(
                nin, nout,
                kernel_size=1,
                bias=False),
            nn.BatchNorm2d(nout),
            activ()
        )

    def __call__(self, x):
        return self.conv(x)


class Encoder(nn.Module):

    def __init__(self, nin, nout, ksize=3, stride=1, pad=1, activ=nn.LeakyReLU):
        super(Encoder, self).__init__()
        self.conv1 = Conv2DBNActiv(nin, nout, ksize, 1, pad, activ=activ)
        self.conv2 = Conv2DBNActiv(nout, nout, ksize, stride, pad, activ=activ)

    def __call__(self, x):
        skip = self.conv1(x)
        h = self.conv2(skip)

        return h, skip


class Decoder(nn.Module):

    def __init__(self, nin, nout, ksize=3, stride=1, pad=1, activ=nn.ReLU, dropout=False):
        super(Decoder, self).__init__()
        self.conv = Conv2DBNActiv(nin, nout, ksize, 1, pad, activ=activ)
        self.dropout = nn.Dropout2d(0.1) if dropout else None

    def __call__(self, x, skip=None):
        x = F.interpolate(x, scale_factor=2, mode='bilinear', align_corners=True)
        if skip is not None:
            skip = crop_center(skip, x)
            x = torch.cat([x, skip], dim=1)
        h = self.conv(x)

        if self.dropout is not None:
            h = self.dropout(h)

        return h


class ASPPModule(nn.Module):

    def __init__(self, nin, nout, dilations=(4, 8, 16, 32, 64), activ=nn.ReLU):
        super(ASPPModule, self).__init__()
        self.conv1 = nn.Sequential(
            nn.AdaptiveAvgPool2d((1, None)),
            Conv2DBNActiv(nin, nin, 1, 1, 0, activ=activ)
        )
        self.conv2 = Conv2DBNActiv(nin, nin, 1, 1, 0, activ=activ)
        self.conv3 = SeperableConv2DBNActiv(
            nin, nin, 3, 1, dilations[0], dilations[0], activ=activ)
        self.conv4 = SeperableConv2DBNActiv(
            nin, nin, 3, 1, dilations[1], dilations[1], activ=activ)
        self.conv5 = SeperableConv2DBNActiv(
            nin, nin, 3, 1, dilations[2], dilations[2], activ=activ)
        self.conv6 = SeperableConv2DBNActiv(
            nin, nin, 3, 1, dilations[2], dilations[2], activ=activ)
        self.conv7 = SeperableConv2DBNActiv(
            nin, nin, 3, 1, dilations[2], dilations[2], activ=activ)
        self.bottleneck = nn.Sequential(
            Conv2DBNActiv(nin * 7, nout, 1, 1, 0, activ=activ),
            nn.Dropout2d(0.1)
        )

    def forward(self, x):
        _, _, h, w = x.size()
        feat1 = F.interpolate(self.conv1(x), size=(h, w), mode='bilinear', align_corners=True)
        feat2 = self.conv2(x)
        feat3 = self.conv3(x)
        feat4 = self.conv4(x)
        feat5 = self.conv5(x)
        feat6 = self.conv6(x)
        feat7 = self.conv7(x)
        out = torch.cat((feat1, feat2, feat3, feat4, feat5, feat6, feat7), dim=1)
        bottle = self.bottleneck(out)
        return bottle

class BaseASPPNet(nn.Module):

    def __init__(self, nin, ch, dilations=(4, 8, 16)):
        super(BaseASPPNet, self).__init__()
        self.enc1 = Encoder(nin, ch, 3, 2, 1)
        self.enc2 = Encoder(ch, ch * 2, 3, 2, 1)
        self.enc3 = Encoder(ch * 2, ch * 4, 3, 2, 1)
        self.enc4 = Encoder(ch * 4, ch * 8, 3, 2, 1)

        self.aspp = ASPPModule(ch * 8, ch * 16, dilations)

        self.dec4 = Decoder(ch * (8 + 16), ch * 8, 3, 1, 1)
        self.dec3 = Decoder(ch * (4 + 8), ch * 4, 3, 1, 1)
        self.dec2 = Decoder(ch * (2 + 4), ch * 2, 3, 1, 1)
        self.dec1 = Decoder(ch * (1 + 2), ch, 3, 1, 1)

    def __call__(self, x):
        h, e1 = self.enc1(x)
        h, e2 = self.enc2(h)
        h, e3 = self.enc3(h)
        h, e4 = self.enc4(h)

        h = self.aspp(h)

        h = self.dec4(h, e4)
        h = self.dec3(h, e3)
        h = self.dec2(h, e2)
        h = self.dec1(h, e1)

        return h


class CascadedASPPNet(nn.Module):

    def __init__(self, n_fft):
        super(CascadedASPPNet, self).__init__()
        self.stg1_low_band_net = BaseASPPNet(2, 64)
        self.stg1_high_band_net = BaseASPPNet(2, 64)

        self.stg2_bridge = Conv2DBNActiv(66, 32, 1, 1, 0)
        self.stg2_full_band_net = BaseASPPNet(32, 64)

        self.stg3_bridge = Conv2DBNActiv(130, 64, 1, 1, 0)
        self.stg3_full_band_net = BaseASPPNet(64, 128)

        self.out = nn.Conv2d(128, 2, 1, bias=False)
        self.aux1_out = nn.Conv2d(64, 2, 1, bias=False)
        self.aux2_out = nn.Conv2d(64, 2, 1, bias=False)

        self.max_bin = n_fft // 2
        self.output_bin = n_fft // 2 + 1

        self.offset = 128

    def forward(self, x, params={}):
        mix = x.detach()
        x = x.clone()

        x = x[:, :, :self.max_bin]

        bandw = x.size()[2] // 2
        aux1 = torch.cat([
            self.stg1_low_band_net(x[:, :, :bandw]),
            self.stg1_high_band_net(x[:, :, bandw:])
        ], dim=2)

        h = torch.cat([x, aux1], dim=1)
        aux2 = self.stg2_full_band_net(self.stg2_bridge(h))

        h = torch.cat([x, aux1, aux2], dim=1)
        h = self.stg3_full_band_net(self.stg3_bridge(h))

        mask = torch.sigmoid(self.out(h))
        mask = F.pad(
            input=mask,
            pad=(0, 0, 0, self.output_bin - mask.size()[2]),
            mode='replicate')
 
        if self.training:
            aux1 = torch.sigmoid(self.aux1_out(aux1))
            aux1 = F.pad(
                input=aux1,
                pad=(0, 0, 0, self.output_bin - aux1.size()[2]),
                mode='replicate')
            aux2 = torch.sigmoid(self.aux2_out(aux2))
            aux2 = F.pad(
                input=aux2,
                pad=(0, 0, 0, self.output_bin - aux2.size()[2]),
                mode='replicate')
            return mask * mix, aux1 * mix, aux2 * mix
        else:
            if params.get('is_vocal_model'):
                return (1.0 - adjust_aggr(mask, params)) * mix  
                
            return adjust_aggr(mask, params) * mix

    def predict(self, x_mag, params={}):
        h = self.forward(x_mag, params)

        if self.offset > 0:
            h = h[:, :, :, self.offset:-self.offset]
            assert h.size()[3] > 0

        return h
    
def make_padding(width, cropsize, offset):
    left = offset
    roi_size = cropsize - left * 2
    if roi_size == 0:
        roi_size = cropsize
    right = roi_size - (width % roi_size) + left

    return left, right, roi_size
    
class VocalRemover(object):

    def __init__(self, model, device, window_size = 512):
        self.model = model
        self.offset = model.offset
        self.device = device
        self.window_size = window_size

    def _execute(self, X_mag_pad, roi_size, n_window, params):
        self.model.eval()
        with torch.no_grad():
            preds = []
            for i in tqdm(range(n_window)):
                start = i * roi_size
                X_mag_window = X_mag_pad[None, :, :, start:start + self.window_size]
                X_mag_window = torch.from_numpy(X_mag_window).to(self.device)

                pred = self.model.predict(X_mag_window, params)

                pred = pred.detach().cpu().numpy()
                preds.append(pred[0])

            pred = np.concatenate(preds, axis=2)

        return pred

    def preprocess(self, X_spec):
        X_mag = np.abs(X_spec)
        X_phase = np.angle(X_spec)

        return X_mag, X_phase

    def inference(self, X_spec, params):
        X_mag, X_phase = self.preprocess(X_spec)

        coef = X_mag.max()
        X_mag_pre = X_mag / coef

        n_frame = X_mag_pre.shape[2]
        pad_l, pad_r, roi_size = make_padding(n_frame, self.window_size, self.offset)
        n_window = int(np.ceil(n_frame / roi_size))

        X_mag_pad = np.pad(X_mag_pre, ((0, 0), (0, 0), (pad_l, pad_r)), mode='constant')

        pred = self._execute(X_mag_pad, roi_size, n_window, params)
        pred = pred[:, :, :n_frame]

        return pred * coef, X_mag, np.exp(1.j * X_phase)

    def inference_tta(self, X_spec, params):
        X_mag, X_phase = self.preprocess(X_spec)

        coef = X_mag.max()
        X_mag_pre = X_mag / coef

        n_frame = X_mag_pre.shape[2]
        pad_l, pad_r, roi_size = make_padding(n_frame, self.window_size, self.offset)
        n_window = int(np.ceil(n_frame / roi_size))

        X_mag_pad = np.pad(X_mag_pre, ((0, 0), (0, 0), (pad_l, pad_r)), mode='constant')

        pred = self._execute(X_mag_pad, roi_size, n_window, params)
        pred = pred[:, :, :n_frame]

        pad_l += roi_size // 2
        pad_r += roi_size // 2
        n_window += 1

        X_mag_pad = np.pad(X_mag_pre, ((0, 0), (0, 0), (pad_l, pad_r)), mode='constant')

        pred_tta = self._execute(X_mag_pad, roi_size, n_window, params)
        pred_tta = pred_tta[:, :, roi_size // 2:]
        pred_tta = pred_tta[:, :, :n_frame]

        return (pred + pred_tta) * 0.5 * coef, X_mag, np.exp(1.j * X_phase)

class ShmartUltimateVocalRemover():
    def __init__(self) -> None:
        self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        self.mp = ModelParameters()
        model = CascadedASPPNet(self.mp.param['bins'] * 2)

        model.load_state_dict(torch.load(get_checkpoint(), map_location='cpu'))
        model.to(self.device)

        self.model = model
        self.vr = VocalRemover(self.model, self.device)

    def __call__(self, wav_path, tta=False, high_end_process='mirroring', aggressiveness=0.5, is_vocal_model=False, sr=22050):
        # adapted from https://github.com/Anjok07/ultimatevocalremovergui/blob/v5-beta-cml/inference.py
        mp = self.mp
        bands_n = len(mp.param['band'])
        bp = mp.param['band'][bands_n]
        input_is_mono = False

        X_spec = {}
        X, sr = librosa.load(wav_path, sr=sr, mono=False, dtype=np.float32, res_type=bp['res_type'])

        if X.ndim == 1:
            input_is_mono = True
            X = np.asarray([X, X])

        X_spec[bands_n] = wave_to_spectrogram(X, bp['hl'], bp['n_fft'], mp, True)
        X_spec[bands_n] = convert_channels(X_spec[bands_n], mp, bands_n)

        if np.max(X[0]) == 0.0:
            print('Empty audio file!')
            raise ValueError('Empty audio file')

        if high_end_process != 'none':
            input_high_end_h = (bp['n_fft']//2 - bp['crop_stop']) + (mp.param['pre_filter_stop'] - mp.param['pre_filter_start'])
            input_high_end = X_spec[bands_n][:, bp['n_fft']//2-input_high_end_h:bp['n_fft']//2, :]    

        for d in range(bands_n - 1, 0, -1):        
            bp = mp.param['band'][d]
            
            X = librosa.resample(X, orig_sr=mp.param['band'][d+1]['sr'], target_sr=bp['sr'], res_type=bp['res_type'])
            X_spec[d] = wave_to_spectrogram(X, bp['hl'], bp['n_fft'], mp, True)
            X_spec[d] = convert_channels(X_spec[d], mp, d)
        
        X_spec = combine_spectrograms(X_spec, mp)

        pd = {
            'aggr_value': aggressiveness,
            'aggr_split_bin': mp.param['band'][1]['crop_stop'],
            'aggr_correction': mp.param.get('aggr_correction'),
            'is_vocal_model': is_vocal_model
        }

        if tta:
            pred, X_mag, X_phase = self.vr.inference_tta(X_spec, pd)
        else:
            pred, X_mag, X_phase = self.vr.inference(X_spec, pd)
        
        ffmpeg_tmp_filename = 'inst_tmp'
        y_spec_m = (pred * X_phase)

        if high_end_process == 'bypass':
            instruments_wave = cmb_spectrogram_to_wave_ffmpeg(y_spec_m, mp, ffmpeg_tmp_filename, input_high_end_h, input_high_end)
        elif high_end_process.startswith('mirroring'):        
            input_high_end_ = mirroring(high_end_process, y_spec_m, input_high_end, mp)
            instruments_wave = cmb_spectrogram_to_wave_ffmpeg(y_spec_m, mp, ffmpeg_tmp_filename, input_high_end_h, input_high_end_)       
        else:
            instruments_wave = cmb_spectrogram_to_wave_ffmpeg(y_spec_m, mp, ffmpeg_tmp_filename)

        if input_is_mono:
            instruments_wave = instruments_wave.mean(axis=1, keepdims=True)

        instruments_wave = torch.from_numpy(instruments_wave).permute(1,0)

        ffmpeg_tmp_filename = 'vocal_tmp'
        v_spec_m = X_spec - y_spec_m

        if high_end_process.startswith('mirroring'):        
            input_high_end_ = mirroring(high_end_process, v_spec_m, input_high_end, mp)
            vocals_wave = cmb_spectrogram_to_wave_ffmpeg(v_spec_m, mp, ffmpeg_tmp_filename, input_high_end_h, input_high_end_)       
        else:        
            vocals_wave = cmb_spectrogram_to_wave_ffmpeg(v_spec_m, mp, ffmpeg_tmp_filename)
        
        if input_is_mono:
            vocals_wave = vocals_wave.mean(axis=1, keepdims=True)

        vocals_wave = torch.from_numpy(vocals_wave).permute(1,0)

        return convert_to_mono_channel(vocals_wave), convert_to_mono_channel(instruments_wave)

if __name__ == '__main__':
    vocals, instruments = ShmartUltimateVocalRemover()('C:/model/1.wav', tta=True)
    save_audio('/content/vr/vocals.wav', vocals)
    save_audio('/content/vr/instruments.wav', instruments)


import os
from dataclasses import dataclass

@dataclass
class ShmartTranscriptionLine():
    name: str
    text: str
    speaker: str = ''
    category: str = ''
    duration: float = -1

    def serialize(self) -> str:
        return f"{'|'.join([self.name, self.text, self.speaker, self.category, str(self.duration)])}\n"
    
    def get_transcription_file(self, wavs_dir_path: str) -> str:
        return os.path.join(wavs_dir_path, f"{self.name}.wav")


import huggingface_hub
import torch
from typing import Tuple

from audclas.tortoise_audio_classifier import TortoiseAudioClassifier
from shttst.audio.utils import save_audio
from shttst.models import ShmartSileroVad, ShmartUltimateVocalRemover

initialized_models= {}

def initialize_model(type):
    model = None
    
    if type == 'classifier':
        model = TortoiseAudioClassifier()
    elif type == 'denoiser':
        model = ShmartUltimateVocalRemover()
    elif type == 'titanet':
        from nemo.collections.asr.models import EncDecSpeakerLabelModel
        model = EncDecSpeakerLabelModel.from_pretrained("nvidia/speakerverification_en_titanet_large", map_location='cpu')
        model.cuda().eval()
    elif type == 'resemblyzer':
        from resemblyzer import VoiceEncoder
        model = VoiceEncoder('cuda', verbose=False)
        model.eval()
    elif type == 'vad':
        model = ShmartSileroVad()
    elif type == 'whisper':
        from faster_whisper import WhisperModel
        model_path = huggingface_hub.snapshot_download("shmart/shmisper-medium-PL")
        model = WhisperModel(model_path, device="cuda", compute_type="float16")
    
    initialized_models[type] = model


def get_model(type):
    if type not in initialized_models:
        initialize_model(type)

    return initialized_models[type]

def classify_audio_file(file_path: str) -> str:
    model: TortoiseAudioClassifier = get_model('classifier')
    return model(file_path)

def split_by_vad(file_path: str, max_duration: int, min_silence: int) -> str:
    model: ShmartSileroVad = get_model('vad')
    return model.vad_process(file_path, min_silence_duration=min_silence, desired_max_speech_duration=max_duration)

def clean_audio_noise(file_path: str) -> str:
    vr_model: ShmartUltimateVocalRemover = get_model('denoiser')
    vocals, instruments = vr_model(file_path, tta=True)
    save_audio(file_path, vocals)

def get_custom_embedding(file_path, type='titanet'):
    if type == 'titanet':
        from nemo.collections.asr.models import EncDecSpeakerLabelModel
        titanet_model: EncDecSpeakerLabelModel = get_model(type)
        return titanet_model.get_embedding(file_path).squeeze(0).cpu()
    elif type == 'resemblyzer':
        from resemblyzer import VoiceEncoder, preprocess_wav
        wav = preprocess_wav(file_path)
        resemblyzer_model: VoiceEncoder = get_model(type)
        return torch.from_numpy(resemblyzer_model.embed_utterance(wav))

def get_transcription(audio_path, options={
    'language': "pl",
    'beam_size': 5,
    'without_timestamps': True,
    'suppress_tokens': [],
    'log_prob_threshold': None,
    'no_speech_threshold': 0.06
}) -> Tuple[str, float]:
    from faster_whisper import WhisperModel
    model: WhisperModel = get_model('whisper')
    result, info = model.transcribe(audio_path, **options)
    try:
      return ' '.join([r.text for r in result]), info.duration
    except:
      return '', -1
    
if __name__ == '__main__':
    get_custom_embedding('\\content\\cyber\\2077\\wavs\\55.wav')


import os
from pathlib import Path
from shttst.audio import make_default_tts_wav, save_audio
from shttst.files import get_audio_files, replace_ext
from tqdm import tqdm

def prepare_tts_wavs(dir_to_lookup: str, destination_dir: str):
    audio_list = get_audio_files(dir_to_lookup)
    
    for file_path in tqdm(audio_list):
        try:
            audio = make_default_tts_wav(file_path)
            relative_dir = str(Path(os.path.relpath(file_path, dir_to_lookup)).parent)
            file_name = f"{replace_ext(file_path)}.wav"
            file_dir = os.path.join(destination_dir, relative_dir)
            os.makedirs(file_dir, exist_ok=True)
            save_audio(os.path.join(file_dir, file_name), audio)
        except Exception as e:
            print(file_path)
            print(e)

if __name__ == '__main__':
    prepare_tts_wavs('D:/audio datasety/gry/skyrim/test', '/content/skyrim')


import librosa
import torch
import torchaudio
import torchaudio.functional as F
import torchaudio.transforms as T

from torch import Tensor
from typing import Tuple

def load_audio(audio_path: str) -> Tuple[Tensor, int]:
    return torchaudio.load(audio_path)

def save_audio(file_path, audio_tensor, target_sr=22050, channels_first=True, encoding="PCM_S", bits_per_sample=16) -> None:
    torchaudio.save(file_path, audio_tensor, target_sr, channels_first=channels_first, encoding=encoding, bits_per_sample=bits_per_sample)

def normalize_volume(audio_tensor, audio_sr, target_volume_db=-24):
    loudness = F.loudness(audio_tensor, audio_sr).item()
    return T.Vol(target_volume_db - loudness, 'db')(audio_tensor)

def resample(audio_tensor, audio_sr, target_sr=22050):
    # # replaced by librosa due to some memory leak (caused crash on google colab for long audio files resampled in loop) - https://github.com/pytorch/audio/issues/2338
    return torch.from_numpy(librosa.resample(audio_tensor.cpu().numpy(), orig_sr=audio_sr, target_sr=target_sr)).float()
    # return F.resample(audio_tensor, audio_sr, target_sr) 

def convert_to_mono_channel(audio_tensor):
    return torch.mean(audio_tensor, 0, True)

def make_default_tts_wav(audio_path) -> Tensor:
    audio, sr = load_audio(audio_path)
    audio = convert_to_mono_channel(audio)
    audio = normalize_volume(audio, sr)
    return resample(audio, sr)


import os
import shutil
from shttst.files import destroy_file, get_files
from shttst.dataset.transcriptions import read_transcriptions_file, write_transcriptions


def scan_dir(dir_path: str):
    return get_files(dir_path, ['csv'], 'metadata')

def check_dataset(metadata_path: str, delete_missing = False):
    wavs_dir_path = os.path.join(os.path.dirname(metadata_path), 'wavs')
    transcriptions = read_transcriptions_file(metadata_path)
    transcribed_files = [x.get_transcription_file(wavs_dir_path) for x in transcriptions]
    if not os.path.exists(wavs_dir_path):
        raise Exception(f'{wavs_dir_path} does not exist, check dataset structure.')
    wavs = get_files(wavs_dir_path, ['wav'])
    transcribed_wavs = [x for x in wavs if x in transcribed_files]
    missing_wavs = [x for x in transcribed_files if x not in wavs]
    missing_transcriptions = [x for x in wavs if x not in transcribed_files]
    if delete_missing:
        if len(missing_wavs):
            transcriptions = [x for x in transcriptions if x.get_transcription_file(wavs_dir_path) not in missing_wavs]
            shutil.move(metadata_path, f'{metadata_path}.bak')
            write_transcriptions(metadata_path, transcriptions)
        for missing_transcription in missing_transcriptions:
            destroy_file(missing_transcription)
    return transcriptions, transcribed_wavs, missing_wavs, missing_transcriptions

if __name__ == '__main__':
    dir_path = '/content/anon'
    for dataset_meta in scan_dir(dir_path):
        try:
            print([len(x) for x in check_dataset(dataset_meta)])
        except Exception as e:
            print(e)
    


import os
from random import shuffle
from shttst.dataset import scan_dir
from shttst.dataset.transcriptions import read_transcriptions_file, write_transcriptions

def prepare_traininig_sets(root_dir: str, test_percent=2):
    metadata_files = scan_dir(root_dir)
    combined_train = []
    combined_test = []
    for meta_path in metadata_files:
        speaker_dir = os.path.relpath(os.path.dirname(meta_path), root_dir)
        transcriptions = read_transcriptions_file(meta_path)
        
        for transcription in transcriptions:
            transcription.name = os.path.join(speaker_dir, 'wavs', f"{transcription.name}.wav")
        
        shuffle(transcriptions)
    
        split_point = int(len(transcriptions) * test_percent / 100)
        combined_test.extend(transcriptions[:split_point])
        combined_train.extend(transcriptions[split_point:])
    
    write_transcriptions(os.path.join(root_dir, 'train.txt'), combined_train)
    write_transcriptions(os.path.join(root_dir, 'test.txt'), combined_test)

if __name__ == '__main__':
    prepare_traininig_sets('/content/todo/rmf')


import os
import torch

from shttst.audio.utils import make_default_tts_wav, resample, save_audio
from shttst.files import destroy_file

class ShmartSileroVad():
    def __init__(self) -> None:
        vad_model, utils = torch.hub.load(repo_or_dir='snakers4/silero-vad',
                                model='silero_vad',
                                force_reload=False,
                                onnx=False)
        self.vad_model = vad_model
        self.get_speech_timestamps = utils[0]
        # self.read_audio = utils[2] # replaced with resample to sr=16000 as silero used faulty torchaudio method (see: resample from shttst.audio.utils)

    def vad_process(self, audio_file, min_silence_duration = 200, initial_max_speech_duration=30, desired_max_speech_duration=14):
        with torch.no_grad():
            tts_sr = 22050
            silero_sr = 16000
            end_frames_fix = int(tts_sr / 12)
            frames_scale = tts_sr / silero_sr
            
            audio_to_split = make_default_tts_wav(audio_file)

            timestamps = self.get_speech_timestamps(
                resample(audio_to_split, tts_sr, silero_sr),
                self.vad_model,
                0.90,  # speech prob threshold
                16000,  # sample rate
                800,  # min speech duration in ms
                initial_max_speech_duration,  # max speech duration in seconds
                min_silence_duration,  # min silence duration
                512,  # window size
                200,  # spech pad ms
            )


            result = []

            for i,t in enumerate(timestamps):
                start = int(t['start'] * frames_scale)
                end = int(t['end'] * frames_scale) + end_frames_fix
                result_audio = audio_to_split[...,start:end]
                if (result_audio.size(0) > 0):
                    name = os.path.splitext(os.path.basename(audio_file))[0]
                    sample_path = os.path.join(os.path.dirname(audio_file), f'{name}_{i}.wav')
                    save_audio(sample_path, result_audio)
                    if (result_audio.size(1) > tts_sr * desired_max_speech_duration):
                        result.extend(self.vad_process(sample_path, min_silence_duration * 0.75, desired_max_speech_duration * 0.75))
                        destroy_file(sample_path)
                    else:
                        result.append(sample_path)

            return result


import os
import shutil
from typing import List
from pathlib import Path

def get_files(dir_path: str, extensions = ['*'], name='*') -> List[str]:
    paths = []
    for ext in extensions:
        paths.extend([str(f) for f in Path(dir_path).rglob(f'{name}.{ext}')])
    return paths

def get_audio_files(dir_path: str) -> List[str]:
    return get_files(dir_path, ['wav', 'ogg', 'flac', 'mp3'])

def read_lines(text_file_path: str) -> List[str]:
    with open(text_file_path, 'r', encoding='utf-8') as fs:
        return [f.rstrip().replace('\n', '') for f in fs.readlines()]

def write_file(text_file_path: str, content: str, mode='w') -> None:
    with open(text_file_path, mode, encoding='utf-8') as fs:
        fs.write(content)

def append_file(text_file_path: str, content: str) -> None:
    write_file(text_file_path, content, 'a')

def get_parent_dir(file_path: str) -> str:
    return os.path.normpath(os.path.join(os.path.dirname(file_path), os.path.pardir))

def move_to_parent(file_path: str):
    if os.path.exists(file_path):
        shutil.move(file_path, os.path.join(get_parent_dir(file_path), os.path.basename(file_path)))

def replace_ext(file_path: str, ext: str):
    assert ext.startswith('.')
    return f"{Path(file_path).stem}{ext}"

def destroy_file(file_path: str) -> None:
    with open(file_path, 'w') as fs:
        fs.close()
    os.remove(file_path)

if __name__ == '__main__':
    # print(get_audio_files('/content/yt')[:5])
    print(read_lines('/content/yt/test.txt')[:5])



from typing import List

from shttst.files import read_lines, write_file
from shttst.dataset import ShmartTranscriptionLine


def read_transcriptions_file(file_path):
    return [
        ShmartTranscriptionLine(a,b,c,d,float(e))
        for a,b,c,d,e in [line.split('|') for line in  read_lines(file_path)]
    ]

def write_transcriptions(file_path, transcriptions: List[ShmartTranscriptionLine]) -> None:
    content = '\n'.join([f"{x.name}|{x.text}|{x.speaker}|{x.category}|{x.duration}" for x in transcriptions])
    write_file(file_path, content)

if __name__ == '__main__':
    write_transcriptions('test.txt', read_transcriptions_file('/content/yt/test.txt')[:5])